@@ -71,7 +71,7 @@ class record(numpy.ndarray):
                         raise TypeError('Record values must be arrays ' \
                             + 'or coercable into arrays.')
                 else:
-                    coerce = copy.deepcopy(input_array)
+                    coerce = input_array
             if isinstance(coerce, numpy.ndarray):
                 obj = numpy.asarray(coerce).view(cls)
             else:
@@ -96,13 +96,23 @@ class record(numpy.ndarray):
         dup._placeholder = None
         return dup
     def __setitem__(self, key, value):
-        if isinstance(key, tuple) and len(key) > 0:
+        if isinstance(value, list) and len(value) == 1:
+            self[key] = value[0]
+        elif isinstance(key, tuple) and len(key) > 0:
             if len(key) > 1:
                 self[key[0]][key[1:]] = value
             else:
                 self[key[0]] = value
         else:
             numpy.ndarray.__setitem__(self, key, value)
+    def __getitem__(self, key):
+        if isinstance(key, tuple) and len(key) > 0:
+            if len(key) > 1:
+                return self[key[0]][key[1:]]
+            else:
+                return self[key[0]]
+        else:
+            return numpy.ndarray.__getitem__(self, key)
     # Selection function
     def select(self):
         if self._variable is not None:
@@ -149,11 +159,19 @@ class record(numpy.ndarray):
         self._num = num
         self._fill()
     def _dehyper(self, shape, hyper):
-        if len(hyper) == 1:
-            self[tuple(shape)] = hyper[0]
+        if len(shape) == len(self.shape):
+            self[tuple(shape)] = hyper
         else:
             for i in xrange(0, len(hyper)):
                 self._dehyper(shape + [i], hyper[i])
+    def _hyper(self, shape, index):
+        if len(shape) > 0:
+            ret = []
+            for i in xrange(0, shape[0]):
+                ret.append(self._hyper(shape[1:], index + [i]))
+            return ret
+        else:
+            return self[tuple(index)]
     def _fill(self):
         if self._placeholder and self._variable is not None:
             with self.selection(self) as selection:
@@ -175,38 +193,39 @@ class record(numpy.ndarray):
                                     [1 for dim in self._variable._dimSizes],
                                 self._variable._tokens['INDEX'],
                                     [0 for dim in self._variable._dimSizes])
-                    try:
-                        (hyper, ) = internal.CDFlib(
-                            internal.GET_,
-                                self._variable._tokens['HYPER'])
-                        self._dehyper([], hyper)
-                    except:
-                        for (index, value) in numpy.ndenumerate(self):
-                            if index is not ():
-                                internal.CDFlib(
-                                    internal.SELECT_,
-                                        self._variable._tokens['INDEX'],
-                                            index)
-                            (value, ) = internal.CDFlib(
-                                internal.GET_,
-                                    self._variable._tokens['DATA'])
-                            self[index] = value
+                    (hyper, ) = internal.CDFlib(
+                        internal.GET_,
+                            self._variable._tokens['HYPER'])
+                    self._dehyper([], hyper)
                     self._placeholder = False
     def _write(self):
         # The variable must have selected itself before calling us.
-        internal.CDFlib(
-            internal.SELECT_,
-                self._variable._tokens['RECORD'],
-                    self._num)
-        for (index, value) in numpy.ndenumerate(self):
-            if index is not ():
+        with self.selection(self) as selection:
+            if selection:
                 internal.CDFlib(
                     internal.SELECT_,
-                        self._variable._tokens['INDEX'],
-                            index)
-            internal.CDFlib(
-                internal.PUT_,
-                    self._variable._tokens['DATA'], value)
+                        self._variable._tokens['RECORD'],
+                            self._num,
+                        self._variable._tokens['RECCOUNT'],
+                            1,
+                        self._variable._tokens['RECINTERVAL'],
+                            1)
+                if self._variable._dimSizes is not None:
+                    internal.CDFlib(
+                        internal.SELECT_,
+                            self._variable._tokens['DIMCOUNTS'],
+                                self._variable._dimSizes,
+                            self._variable._tokens['DIMINTERVALS'],
+                                [1 for dim in self._variable._dimSizes],
+                            self._variable._tokens['INDEX'],
+                                [0 for dim in self._variable._dimSizes])
+                hyper = self._hyper(list(self.shape), [])
+                if not isinstance(hyper, list):
+                    hyper = [hyper]
+                internal.CDFlib(
+                    internal.PUT_,
+                        self._variable._tokens['HYPER'],
+                            hyper)
     def indices(self):
         # This is a generator function which will iterate over all
         # indices of this variable.  It is suitable for calls to
@@ -243,12 +262,6 @@ class variable(list):
                 self.extend(value)
             else:
                 self.extend([value])
-    def __del__(self):
-        del self.attr
-        list.__del__(self)
-        internal.CDFlib(
-            internal.CLOSE_,
-                self._tokens['SELECT_VARIABLE'])
 
     # Selection function
     def select(self):
@@ -779,7 +792,8 @@ class archive(dict):
                 self._filenames.remove(filename)
             self._filenames.append(filename)
         with self.selection(self, filename) as selection:
-            self._save()
+            if selection:
+                self._save()
     def _save(self):
         # If there have been things removed from the archive then we
         # need to remove them on disk.  Order matters here, because
